@@ -7,10 +7,10 @@
 #ifndef CORE_FPDFAPI_CMAPS_JAPAN1_CMAPS_JAPAN1_H_
 #define CORE_FPDFAPI_CMAPS_JAPAN1_CMAPS_JAPAN1_H_
 
-#include <stddef.h>
 #include <stdint.h>
 
 #include "core/fpdfapi/cmaps/fpdf_cmaps.h"
+#include "core/fxcrt/span.h"
 
 namespace fxcmap {
 
@@ -36,8 +36,8 @@ extern const uint16_t kUniJIS_UTF16_H_0[];
 extern const uint16_t kUniJIS_UTF16_H_0_DWord[];
 extern const uint16_t kUniJIS_UTF16_V_0[];
 extern const uint16_t kJapan1CID2Unicode_4[15444];
-extern const CMap kJapan1_cmaps[];
-extern const size_t kJapan1_cmaps_size;
+
+extern const pdfium::span<const CMap> kJapan1_cmaps_span;
 
 }  // namespace fxcmap
 
