@@ -8,7 +8,7 @@
 
 #include <algorithm>
 
-#include "third_party/base/check.h"
+#include "core/fxcrt/check.h"
 
 namespace fxcmap {
 
