@@ -28,15 +28,15 @@
 #include "core/fpdfapi/parser/cpdf_syntax_parser.h"
 #include "core/fpdfapi/parser/fpdf_parser_utility.h"
 #include "core/fxcrt/autorestorer.h"
+#include "core/fxcrt/check.h"
+#include "core/fxcrt/check_op.h"
+#include "core/fxcrt/containers/contains.h"
 #include "core/fxcrt/data_vector.h"
 #include "core/fxcrt/fx_extension.h"
 #include "core/fxcrt/fx_safe_types.h"
+#include "core/fxcrt/notreached.h"
 #include "core/fxcrt/scoped_set_insertion.h"
-#include "third_party/base/check.h"
-#include "third_party/base/check_op.h"
-#include "third_party/base/containers/contains.h"
-#include "third_party/base/containers/span.h"
-#include "third_party/base/notreached.h"
+#include "core/fxcrt/span.h"
 
 using ObjectType = CPDF_CrossRefTable::ObjectType;
 using ObjectInfo = CPDF_CrossRefTable::ObjectInfo;
@@ -55,10 +55,10 @@ constexpr FX_FILESIZE kPDFHeaderSize = 9;
 // dictionary.
 constexpr size_t kMinFieldCount = 3;
 
-// V4 trailers are inline.
-constexpr uint32_t kNoV4TrailerObjectNumber = 0;
+// Trailers are inline.
+constexpr uint32_t kNoTrailerObjectNumber = 0;
 
-struct CrossRefV5IndexEntry {
+struct CrossRefStreamIndexEntry {
   uint32_t start_obj_num;
   uint32_t obj_count;
 };
@@ -104,9 +104,10 @@ uint32_t GetThirdXRefStreamEntry(pdfium::span<const uint8_t> entry_span,
       entry_span.subspan(field_widths[0] + field_widths[1], field_widths[2]));
 }
 
-std::vector<CrossRefV5IndexEntry> GetCrossRefV5Indices(const CPDF_Array* array,
-                                                       uint32_t size) {
-  std::vector<CrossRefV5IndexEntry> indices;
+std::vector<CrossRefStreamIndexEntry> GetCrossRefStreamIndices(
+    const CPDF_Array* array,
+    uint32_t size) {
+  std::vector<CrossRefStreamIndexEntry> indices;
   if (array) {
     for (size_t i = 0; i < array->size() / 2; i++) {
       RetainPtr<const CPDF_Number> pStartNumObj = array->GetNumberAt(i * 2);
@@ -203,7 +204,7 @@ bool CPDF_Parser::IsObjectFree(uint32_t objnum) const {
 }
 
 bool CPDF_Parser::InitSyntaxParser(RetainPtr<CPDF_ReadValidator> validator) {
-  const absl::optional<FX_FILESIZE> header_offset = GetHeaderOffset(validator);
+  const std::optional<FX_FILESIZE> header_offset = GetHeaderOffset(validator);
   if (!header_offset.has_value())
     return false;
   if (validator->GetSize() < header_offset.value() + kPDFHeaderSize)
@@ -249,8 +250,8 @@ CPDF_Parser::Error CPDF_Parser::StartParseInternal() {
 
   m_LastXRefOffset = ParseStartXRef();
   if (m_LastXRefOffset >= kPDFHeaderSize) {
-    if (!LoadAllCrossRefV4(m_LastXRefOffset) &&
-        !LoadAllCrossRefV5(m_LastXRefOffset)) {
+    if (!LoadAllCrossRefTables(m_LastXRefOffset) &&
+        !LoadAllCrossRefStreams(m_LastXRefOffset)) {
       if (!RebuildCrossRef())
         return FORMAT_ERROR;
 
@@ -351,7 +352,7 @@ void CPDF_Parser::ReleaseEncryptHandler() {
 // In reality, we rarely see well-formed cross references don't match
 // with the objects. crbug/602650 showed a case where object numbers
 // in the cross reference table are all off by one.
-bool CPDF_Parser::VerifyCrossRefV4() {
+bool CPDF_Parser::VerifyCrossRefTable() {
   for (const auto& it : m_CrossRefTable->objects_info()) {
     if (it.second.pos <= 0)
       continue;
@@ -371,76 +372,64 @@ bool CPDF_Parser::VerifyCrossRefV4() {
   return true;
 }
 
-bool CPDF_Parser::LoadAllCrossRefV4(FX_FILESIZE xref_offset) {
-  if (!LoadCrossRefV4(xref_offset, true))
+bool CPDF_Parser::LoadAllCrossRefTables(FX_FILESIZE xref_offset) {
+  if (!LoadCrossRefTable(xref_offset, /*skip=*/true)) {
     return false;
+  }
 
-  RetainPtr<CPDF_Dictionary> trailer = LoadTrailerV4();
+  RetainPtr<CPDF_Dictionary> trailer = LoadTrailer();
   if (!trailer)
     return false;
 
-  m_CrossRefTable->SetTrailer(std::move(trailer), kNoV4TrailerObjectNumber);
+  m_CrossRefTable->SetTrailer(std::move(trailer), kNoTrailerObjectNumber);
   const int32_t xrefsize = GetTrailer()->GetDirectIntegerFor("Size");
   if (xrefsize > 0 && xrefsize <= kMaxXRefSize)
     m_CrossRefTable->SetObjectMapSize(xrefsize);
 
   FX_FILESIZE xref_stm = GetTrailer()->GetDirectIntegerFor("XRefStm");
-  std::vector<FX_FILESIZE> xref_stream_list{xref_stm};
-  std::vector<FX_FILESIZE> xref_list{xref_offset};
-  std::set<FX_FILESIZE> seen_xref_offset{xref_offset};
-
-  // When the trailer doesn't have Prev entry or Prev entry value is not
-  // numerical, GetDirectInteger() returns 0. Loading will end.
-  xref_offset = GetTrailer()->GetDirectIntegerFor("Prev");
-  while (xref_offset > 0) {
-    // Check for circular references.
-    if (pdfium::Contains(seen_xref_offset, xref_offset))
-      return false;
-
-    seen_xref_offset.insert(xref_offset);
-    xref_list.insert(xref_list.begin(), xref_offset);
-
-    // SLOW ...
-    LoadCrossRefV4(xref_offset, true);
-
-    RetainPtr<CPDF_Dictionary> pDict(LoadTrailerV4());
-    if (!pDict)
-      return false;
-
-    xref_offset = pDict->GetDirectIntegerFor("Prev");
-    xref_stm = pDict->GetIntegerFor("XRefStm");
-    xref_stream_list.insert(xref_stream_list.begin(), xref_stm);
-
-    // SLOW ...
-    m_CrossRefTable = CPDF_CrossRefTable::MergeUp(
-        std::make_unique<CPDF_CrossRefTable>(std::move(pDict),
-                                             kNoV4TrailerObjectNumber),
-        std::move(m_CrossRefTable));
-  }
-
-  // Traverse the xref data structures from oldest to newest. So entries from
-  // later iterations should overwrite existing entries.
-  for (size_t i = 0; i < xref_list.size(); ++i) {
-    if (xref_list[i] > 0 && !LoadCrossRefV4(xref_list[i], false))
-      return false;
-
-    if (xref_stream_list[i] > 0 &&
-        !LoadCrossRefV5(&xref_stream_list[i], /*is_main_xref=*/false,
-                        /*overwrite_existing=*/true)) {
+  if (xref_stm > 0) {
+    // In hybrid-reference files, use /Prev from the cross-reference stream and
+    // ignore the /Prev in the trailer.
+    // See ISO 32000-1:2008 spec, table 17.
+    if (!LoadAllSecondaryCrossRefStreams(xref_stm)) {
       return false;
     }
 
-    if (i == 0 && !VerifyCrossRefV4())
+    // Then load the reference table in the trailer.
+    return LoadCrossRefTable(xref_offset, /*skip=*/false);
+  }
+
+  // Otherwise, follow the /Prev from the trailer.
+  std::vector<FX_FILESIZE> xref_list{xref_offset};
+  std::vector<FX_FILESIZE> xref_stream_list{xref_stm};
+  if (!FindAllCrossReferenceTablesAndStream(xref_offset, xref_list,
+                                            xref_stream_list)) {
+    return false;
+  }
+
+  for (size_t i = 0; i < xref_list.size(); ++i) {
+    if (xref_list[i] > 0 && !LoadCrossRefTable(xref_list[i], /*skip=*/false)) {
       return false;
+    }
+
+    if (xref_stream_list[i] > 0 &&
+        !LoadCrossRefStream(&xref_stream_list[i], /*is_main_xref=*/false)) {
+      return false;
+    }
+
+    if (i == 0 && !VerifyCrossRefTable()) {
+      return false;
+    }
   }
   return true;
 }
 
-bool CPDF_Parser::LoadLinearizedAllCrossRefV4(FX_FILESIZE main_xref_offset) {
-  if (!LoadCrossRefV4(main_xref_offset, false))
+bool CPDF_Parser::LoadLinearizedAllCrossRefTable(FX_FILESIZE main_xref_offset) {
+  if (!LoadCrossRefTable(main_xref_offset, /*skip=*/false)) {
     return false;
+  }
 
-  RetainPtr<CPDF_Dictionary> main_trailer = LoadTrailerV4();
+  RetainPtr<CPDF_Dictionary> main_trailer = LoadTrailer();
   if (!main_trailer)
     return false;
 
@@ -452,60 +441,37 @@ bool CPDF_Parser::LoadLinearizedAllCrossRefV4(FX_FILESIZE main_xref_offset) {
   // first-page trailer, as the caller already did that and passed it in as
   // |main_xref_offset|.
   FX_FILESIZE xref_stm = GetTrailer()->GetDirectIntegerFor("XRefStm");
-  std::vector<FX_FILESIZE> xref_stream_list{xref_stm};
   std::vector<FX_FILESIZE> xref_list{main_xref_offset};
-  std::set<FX_FILESIZE> seen_xref_offset{main_xref_offset};
+  std::vector<FX_FILESIZE> xref_stream_list{xref_stm};
 
-  // Merge the trailers.
+  // Merge the trailers. Now GetTrailer() returns the merged trailer, where
+  // /Prev is from the main-trailer.
   m_CrossRefTable = CPDF_CrossRefTable::MergeUp(
       std::make_unique<CPDF_CrossRefTable>(std::move(main_trailer),
-                                           kNoV4TrailerObjectNumber),
+                                           kNoTrailerObjectNumber),
       std::move(m_CrossRefTable));
 
-  // Now GetTrailer() returns the merged trailer, where /Prev is from the
-  // main-trailer.
-  FX_FILESIZE xref_offset = GetTrailer()->GetDirectIntegerFor("Prev");
-  while (xref_offset > 0) {
-    // Check for circular references.
-    if (pdfium::Contains(seen_xref_offset, xref_offset))
+  // Ignore /Prev for hybrid-reference files.
+  // See ISO 32000-1:2008 spec, table 17.
+  if (!xref_stm) {
+    if (!FindAllCrossReferenceTablesAndStream(main_xref_offset, xref_list,
+                                              xref_stream_list)) {
       return false;
-
-    seen_xref_offset.insert(xref_offset);
-    xref_list.insert(xref_list.begin(), xref_offset);
-
-    // SLOW ...
-    LoadCrossRefV4(xref_offset, true);
-
-    RetainPtr<CPDF_Dictionary> pDict(LoadTrailerV4());
-    if (!pDict)
-      return false;
-
-    xref_offset = pDict->GetDirectIntegerFor("Prev");
-    xref_stm = pDict->GetIntegerFor("XRefStm");
-    xref_stream_list.insert(xref_stream_list.begin(), xref_stm);
-
-    // SLOW ...
-    m_CrossRefTable = CPDF_CrossRefTable::MergeUp(
-        std::make_unique<CPDF_CrossRefTable>(std::move(pDict),
-                                             kNoV4TrailerObjectNumber),
-        std::move(m_CrossRefTable));
+    }
   }
 
   if (xref_stream_list[0] > 0 &&
-      !LoadCrossRefV5(&xref_stream_list[0], /*is_main_xref=*/false,
-                      /*overwrite_existing=*/true)) {
+      !LoadCrossRefStream(&xref_stream_list[0], /*is_main_xref=*/false)) {
     return false;
   }
 
-  // Traverse the xref data structures from oldest to newest. So entries from
-  // later iterations should overwrite existing entries.
   for (size_t i = 1; i < xref_list.size(); ++i) {
-    if (xref_list[i] > 0 && !LoadCrossRefV4(xref_list[i], false))
+    if (xref_list[i] > 0 && !LoadCrossRefTable(xref_list[i], /*skip=*/false)) {
       return false;
+    }
 
     if (xref_stream_list[i] > 0 &&
-        !LoadCrossRefV5(&xref_stream_list[i], /*is_main_xref=*/false,
-                        /*overwrite_existing=*/true)) {
+        !LoadCrossRefStream(&xref_stream_list[i], /*is_main_xref=*/false)) {
       return false;
     }
   }
@@ -597,7 +563,8 @@ bool CPDF_Parser::ParseAndAppendCrossRefSubsectionData(
   return true;
 }
 
-bool CPDF_Parser::ParseCrossRefV4(std::vector<CrossRefObjData>* out_objects) {
+bool CPDF_Parser::ParseCrossRefTable(
+    std::vector<CrossRefObjData>* out_objects) {
   if (out_objects)
     out_objects->clear();
 
@@ -633,11 +600,12 @@ bool CPDF_Parser::ParseCrossRefV4(std::vector<CrossRefObjData>* out_objects) {
   return true;
 }
 
-bool CPDF_Parser::LoadCrossRefV4(FX_FILESIZE pos, bool bSkip) {
+bool CPDF_Parser::LoadCrossRefTable(FX_FILESIZE pos, bool skip) {
   m_pSyntax->SetPos(pos);
   std::vector<CrossRefObjData> objects;
-  if (!ParseCrossRefV4(bSkip ? nullptr : &objects))
+  if (!ParseCrossRefTable(skip ? nullptr : &objects)) {
     return false;
+  }
 
   MergeCrossRefObjectsData(objects);
   return true;
@@ -652,39 +620,100 @@ void CPDF_Parser::MergeCrossRefObjectsData(
           m_CrossRefTable->SetFree(obj.obj_num);
         break;
       case ObjectType::kNormal:
-      case ObjectType::kObjStream:
-        m_CrossRefTable->AddNormal(obj.obj_num, obj.info.gennum, obj.info.pos);
+        m_CrossRefTable->AddNormal(obj.obj_num, obj.info.gennum,
+                                   obj.info.is_object_stream_flag,
+                                   obj.info.pos);
         break;
       case ObjectType::kCompressed:
         m_CrossRefTable->AddCompressed(obj.obj_num, obj.info.archive.obj_num,
                                        obj.info.archive.obj_index);
         break;
+      case ObjectType::kNull:
+        // Ignored.
+        break;
     }
   }
 }
 
-bool CPDF_Parser::LoadAllCrossRefV5(FX_FILESIZE xref_offset) {
-  if (!LoadCrossRefV5(&xref_offset, /*is_main_xref=*/true,
-                      /*overwrite_existing=*/false)) {
+bool CPDF_Parser::LoadAllCrossRefStreams(FX_FILESIZE xref_offset) {
+  if (!LoadCrossRefStream(&xref_offset, /*is_main_xref=*/true)) {
     return false;
   }
 
-  // Traverse the xref objects from newest to older. So entries from later
-  // iterations should not overwrite existing entries.
+  if (!LoadAllSecondaryCrossRefStreams(xref_offset)) {
+    return false;
+  }
+
+  m_ObjectStreamMap.clear();
+  m_bXRefStream = true;
+  return true;
+}
+
+bool CPDF_Parser::LoadAllSecondaryCrossRefStreams(FX_FILESIZE xref_offset) {
   std::set<FX_FILESIZE> seen_xref_offset;
   while (xref_offset > 0) {
     seen_xref_offset.insert(xref_offset);
-    if (!LoadCrossRefV5(&xref_offset, /*is_main_xref=*/false,
-                        /*overwrite_existing=*/false)) {
-      return false;
+    FX_FILESIZE save_xref_offset = xref_offset;
+    if (!LoadCrossRefStream(&xref_offset, /*is_main_xref=*/false)) {
+      // If a cross-reference stream failed to load at `xref_offset`, try
+      // loading a cross-reference table at the same location. Use
+      // `save_xref_offset` instead of `xref_offset`, as `xref_offset` may have
+      // changed.
+      if (!LoadCrossRefTable(save_xref_offset, /*skip=*/false)) {
+        return false;
+      }
+
+      RetainPtr<CPDF_Dictionary> trailer_dict = LoadTrailer();
+      if (!trailer_dict) {
+        return false;
+      }
+      xref_offset = trailer_dict->GetDirectIntegerFor("Prev");
     }
 
     // Check for circular references.
-    if (pdfium::Contains(seen_xref_offset, xref_offset))
+    if (pdfium::Contains(seen_xref_offset, xref_offset)) {
       return false;
+    }
+  }
+  return true;
+}
+
+bool CPDF_Parser::FindAllCrossReferenceTablesAndStream(
+    FX_FILESIZE main_xref_offset,
+    std::vector<FX_FILESIZE>& xref_list,
+    std::vector<FX_FILESIZE>& xref_stream_list) {
+  std::set<FX_FILESIZE> seen_xref_offset{main_xref_offset};
+
+  // When the trailer doesn't have Prev entry or Prev entry value is not
+  // numerical, GetDirectInteger() returns 0. Loading will end.
+  FX_FILESIZE xref_offset = GetTrailer()->GetDirectIntegerFor("Prev");
+  while (xref_offset > 0) {
+    // Check for circular references.
+    if (pdfium::Contains(seen_xref_offset, xref_offset)) {
+      return false;
+    }
+
+    seen_xref_offset.insert(xref_offset);
+    xref_list.insert(xref_list.begin(), xref_offset);
+
+    // SLOW ...
+    LoadCrossRefTable(xref_offset, /*skip=*/true);
+
+    RetainPtr<CPDF_Dictionary> trailer_dict = LoadTrailer();
+    if (!trailer_dict) {
+      return false;
+    }
+
+    xref_offset = trailer_dict->GetDirectIntegerFor("Prev");
+    xref_stream_list.insert(xref_stream_list.begin(),
+                            trailer_dict->GetIntegerFor("XRefStm"));
+
+    // SLOW ...
+    m_CrossRefTable = CPDF_CrossRefTable::MergeUp(
+        std::make_unique<CPDF_CrossRefTable>(std::move(trailer_dict),
+                                             kNoTrailerObjectNumber),
+        std::move(m_CrossRefTable));
   }
-  m_ObjectStreamMap.clear();
-  m_bXRefStream = true;
   return true;
 }
 
@@ -745,7 +774,8 @@ bool CPDF_Parser::RebuildCrossRef() {
       }
 
       if (obj_num < kMaxObjectNumber) {
-        cross_ref_table->AddNormal(obj_num, gen_num, obj_pos);
+        cross_ref_table->AddNormal(obj_num, gen_num, /*is_object_stream=*/false,
+                                   obj_pos);
         const auto object_stream =
             CPDF_ObjectStream::Create(std::move(pStream));
         if (object_stream) {
@@ -769,9 +799,7 @@ bool CPDF_Parser::RebuildCrossRef() {
   return GetTrailer() && !m_CrossRefTable->objects_info().empty();
 }
 
-bool CPDF_Parser::LoadCrossRefV5(FX_FILESIZE* pos,
-                                 bool is_main_xref,
-                                 bool overwrite_existing) {
+bool CPDF_Parser::LoadCrossRefStream(FX_FILESIZE* pos, bool is_main_xref) {
   RetainPtr<const CPDF_Stream> pStream =
       ToStream(ParseIndirectObjectAt(*pos, 0));
   if (!pStream || !pStream->GetObjNum()) {
@@ -800,8 +828,8 @@ bool CPDF_Parser::LoadCrossRefV5(FX_FILESIZE* pos,
         std::move(new_cross_ref_table), std::move(m_CrossRefTable));
   }
 
-  std::vector<CrossRefV5IndexEntry> indices =
-      GetCrossRefV5Indices(pDict->GetArrayFor("Index").Get(), size);
+  std::vector<CrossRefStreamIndexEntry> indices =
+      GetCrossRefStreamIndices(pDict->GetArrayFor("Index").Get(), size);
 
   std::vector<uint32_t> field_widths =
       GetFieldWidths(pDict->GetArrayFor("W").Get());
@@ -836,10 +864,10 @@ bool CPDF_Parser::LoadCrossRefV5(FX_FILESIZE* pos,
     }
 
     // Until SetObjectMapSize() below has been called by a prior loop iteration,
-    // `current_size` is based on the /Size value parsed in LoadCrossRefV5().
-    // PDFs may not always have the correct /Size. In this case, other PDF
-    // implementations ignore the incorrect size, and PDFium also ignores
-    // incorrect size in trailers for V4 xrefs.
+    // `current_size` is based on the /Size value parsed in
+    // LoadCrossRefStream(). PDFs may not always have the correct /Size. In this
+    // case, other PDF implementations ignore the incorrect size, and PDFium
+    // also ignores incorrect size in trailers for cross reference tables.
     const uint32_t current_size =
         m_CrossRefTable->objects_info().empty() ? 0 : GetLastObjNum() + 1;
     // So allow `new_size` to be greater than `current_size`, but avoid going
@@ -857,8 +885,8 @@ bool CPDF_Parser::LoadCrossRefV5(FX_FILESIZE* pos,
         break;
       }
 
-      ProcessCrossRefV5Entry(seg_span.subspan(i * total_width, total_width),
-                             field_widths, obj_num, overwrite_existing);
+      ProcessCrossRefStreamEntry(seg_span.subspan(i * total_width, total_width),
+                                 field_widths, obj_num);
     }
 
     segindex += index.obj_count;
@@ -866,11 +894,10 @@ bool CPDF_Parser::LoadCrossRefV5(FX_FILESIZE* pos,
   return true;
 }
 
-void CPDF_Parser::ProcessCrossRefV5Entry(
+void CPDF_Parser::ProcessCrossRefStreamEntry(
     pdfium::span<const uint8_t> entry_span,
     pdfium::span<const uint32_t> field_widths,
-    uint32_t obj_num,
-    bool overwrite_existing) {
+    uint32_t obj_num) {
   DCHECK_GE(field_widths.size(), kMinFieldCount);
   ObjectType type;
   if (field_widths[0]) {
@@ -890,12 +917,14 @@ void CPDF_Parser::ProcessCrossRefV5Entry(
   const ObjectType existing_type = GetObjectType(obj_num);
   if (existing_type == ObjectType::kNull) {
     const uint32_t offset = GetSecondXRefStreamEntry(entry_span, field_widths);
-    if (pdfium::base::IsValueInRangeForNumericType<FX_FILESIZE>(offset))
-      m_CrossRefTable->AddNormal(obj_num, 0, offset);
+    if (pdfium::IsValueInRangeForNumericType<FX_FILESIZE>(offset)) {
+      m_CrossRefTable->AddNormal(obj_num, 0, /*is_object_stream=*/false,
+                                 offset);
+    }
     return;
   }
 
-  if (!overwrite_existing && existing_type != ObjectType::kFree) {
+  if (existing_type != ObjectType::kFree) {
     return;
   }
 
@@ -906,8 +935,10 @@ void CPDF_Parser::ProcessCrossRefV5Entry(
 
   if (type == ObjectType::kNormal) {
     const uint32_t offset = GetSecondXRefStreamEntry(entry_span, field_widths);
-    if (pdfium::base::IsValueInRangeForNumericType<FX_FILESIZE>(offset))
-      m_CrossRefTable->AddNormal(obj_num, 0, offset);
+    if (pdfium::IsValueInRangeForNumericType<FX_FILESIZE>(offset)) {
+      m_CrossRefTable->AddNormal(obj_num, 0, /*is_object_stream=*/false,
+                                 offset);
+    }
     return;
   }
 
@@ -1028,7 +1059,7 @@ const CPDF_ObjectStream* CPDF_Parser::GetObjectStream(uint32_t object_number) {
     return it->second.get();
 
   const auto* info = m_CrossRefTable->GetObjectInfo(object_number);
-  if (!info || info->type != ObjectType::kObjStream) {
+  if (!info || !info->is_object_stream_flag) {
     return nullptr;
   }
 
@@ -1086,7 +1117,7 @@ void CPDF_Parser::SetLinearizedHeaderForTesting(
   m_pLinearized = std::move(pLinearized);
 }
 
-RetainPtr<CPDF_Dictionary> CPDF_Parser::LoadTrailerV4() {
+RetainPtr<CPDF_Dictionary> CPDF_Parser::LoadTrailer() {
   if (m_pSyntax->GetKeyword() != "trailer")
     return nullptr;
 
@@ -1123,21 +1154,22 @@ CPDF_Parser::Error CPDF_Parser::StartLinearizedParse(
 
   m_LastXRefOffset = m_pLinearized->GetLastXRefOffset();
   FX_FILESIZE dwFirstXRefOffset = m_LastXRefOffset;
-  bool bLoadV4 = LoadCrossRefV4(dwFirstXRefOffset, false);
-  if (!bLoadV4 && !LoadCrossRefV5(&dwFirstXRefOffset, /*is_main_xref=*/true,
-                                  /*overwrite_existing=*/false)) {
+  const bool loaded_xref_table =
+      LoadCrossRefTable(dwFirstXRefOffset, /*skip=*/false);
+  if (!loaded_xref_table &&
+      !LoadCrossRefStream(&dwFirstXRefOffset, /*is_main_xref=*/true)) {
     if (!RebuildCrossRef())
       return FORMAT_ERROR;
 
     m_bXRefTableRebuilt = true;
     m_LastXRefOffset = 0;
   }
-  if (bLoadV4) {
-    RetainPtr<CPDF_Dictionary> trailer = LoadTrailerV4();
+  if (loaded_xref_table) {
+    RetainPtr<CPDF_Dictionary> trailer = LoadTrailer();
     if (!trailer)
       return SUCCESS;
 
-    m_CrossRefTable->SetTrailer(std::move(trailer), kNoV4TrailerObjectNumber);
+    m_CrossRefTable->SetTrailer(std::move(trailer), kNoTrailerObjectNumber);
     const int32_t xrefsize = GetTrailer()->GetDirectIntegerFor("Size");
     if (xrefsize > 0) {
       // Check if `xrefsize` is correct. If it is incorrect, give up and rebuild
@@ -1189,20 +1221,17 @@ CPDF_Parser::Error CPDF_Parser::StartLinearizedParse(
   return SUCCESS;
 }
 
-bool CPDF_Parser::LoadLinearizedAllCrossRefV5(FX_FILESIZE main_xref_offset) {
+bool CPDF_Parser::LoadLinearizedAllCrossRefStream(
+    FX_FILESIZE main_xref_offset) {
   FX_FILESIZE xref_offset = main_xref_offset;
-  if (!LoadCrossRefV5(&xref_offset, /*is_main_xref=*/false,
-                      /*overwrite_existing=*/false)) {
+  if (!LoadCrossRefStream(&xref_offset, /*is_main_xref=*/false)) {
     return false;
   }
 
-  // Traverse the xref objects from newest to older. So entries from later
-  // iterations should not overwrite existing entries.
   std::set<FX_FILESIZE> seen_xref_offset;
   while (xref_offset) {
     seen_xref_offset.insert(xref_offset);
-    if (!LoadCrossRefV5(&xref_offset, /*is_main_xref=*/false,
-                        /*overwrite_existing=*/false)) {
+    if (!LoadCrossRefStream(&xref_offset, /*is_main_xref=*/false)) {
       return false;
     }
 
@@ -1228,8 +1257,8 @@ CPDF_Parser::Error CPDF_Parser::LoadLinearizedMainXRefTable() {
   m_MetadataObjnum = 0;
   m_ObjectStreamMap.clear();
 
-  if (!LoadLinearizedAllCrossRefV4(main_xref_offset) &&
-      !LoadLinearizedAllCrossRefV5(main_xref_offset)) {
+  if (!LoadLinearizedAllCrossRefTable(main_xref_offset) &&
+      !LoadLinearizedAllCrossRefStream(main_xref_offset)) {
     m_LastXRefOffset = 0;
     return FORMAT_ERROR;
   }
