@@ -11,7 +11,9 @@
 #include "core/fpdfapi/page/cpdf_transferfunc.h"
 #include "core/fpdfapi/parser/cpdf_array.h"
 #include "core/fpdfapi/parser/cpdf_dictionary.h"
+#include "core/fpdfapi/parser/cpdf_indirect_object_holder.h"
 #include "core/fpdfapi/parser/cpdf_number.h"
+#include "core/fpdfapi/parser/cpdf_reference.h"
 #include "core/fpdfapi/parser/cpdf_stream.h"
 #include "core/fxcrt/data_vector.h"
 #include "testing/gtest/include/gtest/gtest.h"
@@ -70,7 +72,8 @@ constexpr uint8_t kExpectedType4FunctionSamples[] = {
     26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26,
     26, 26, 26, 26, 26, 26, 26, 26, 26};
 
-RetainPtr<CPDF_Stream> CreateType0FunctionStream() {
+RetainPtr<CPDF_Reference> CreateType0FunctionStreamReference(
+    CPDF_IndirectObjectHolder& holder) {
   auto func_dict = pdfium::MakeRetain<CPDF_Dictionary>();
   func_dict->SetNewFor<CPDF_Number>("FunctionType", 0);
   func_dict->SetNewFor<CPDF_Number>("BitsPerSample", 8);
@@ -87,9 +90,10 @@ RetainPtr<CPDF_Stream> CreateType0FunctionStream() {
   size_array->AppendNew<CPDF_Number>(4);
 
   static constexpr uint8_t kContents[] = "1234";
-  return pdfium::MakeRetain<CPDF_Stream>(
+  auto stream = holder.NewIndirect<CPDF_Stream>(
       DataVector<uint8_t>(std::begin(kContents), std::end(kContents)),
       std::move(func_dict));
+  return pdfium::MakeRetain<CPDF_Reference>(&holder, stream->GetObjNum());
 }
 
 RetainPtr<CPDF_Dictionary> CreateType2FunctionDict() {
@@ -114,7 +118,8 @@ RetainPtr<CPDF_Dictionary> CreateType2FunctionDict() {
   return func_dict;
 }
 
-RetainPtr<CPDF_Stream> CreateType4FunctionStream() {
+RetainPtr<CPDF_Reference> CreateType4FunctionStreamReference(
+    CPDF_IndirectObjectHolder& holder) {
   auto func_dict = pdfium::MakeRetain<CPDF_Dictionary>();
   func_dict->SetNewFor<CPDF_Number>("FunctionType", 4);
 
@@ -127,9 +132,10 @@ RetainPtr<CPDF_Stream> CreateType4FunctionStream() {
   range_array->AppendNew<CPDF_Number>(1);
 
   static constexpr uint8_t kContents[] = "{ 360 mul sin 2 div }";
-  return pdfium::MakeRetain<CPDF_Stream>(
+  auto stream = holder.NewIndirect<CPDF_Stream>(
       DataVector<uint8_t>(std::begin(kContents), std::end(kContents)),
       std::move(func_dict));
+  return pdfium::MakeRetain<CPDF_Reference>(&holder, stream->GetObjNum());
 }
 
 RetainPtr<CPDF_Stream> CreateBadType4FunctionStream() {
@@ -194,10 +200,11 @@ TEST(CPDF_DocRenderDataTest, TransferFunctionOne) {
 }
 
 TEST(CPDF_DocRenderDataTest, TransferFunctionArray) {
+  CPDF_IndirectObjectHolder holder;
   auto func_array = pdfium::MakeRetain<CPDF_Array>();
-  func_array->Append(CreateType0FunctionStream());
+  func_array->Append(CreateType0FunctionStreamReference(holder));
   func_array->Append(CreateType2FunctionDict());
-  func_array->Append(CreateType4FunctionStream());
+  func_array->Append(CreateType4FunctionStreamReference(holder));
 
   TestDocRenderData render_data;
   auto func = render_data.CreateTransferFuncForTesting(func_array);
@@ -247,10 +254,15 @@ TEST(CPDF_DocRenderDataTest, BadTransferFunctions) {
   }
 
   {
+    CPDF_IndirectObjectHolder holder;
     auto func_array = pdfium::MakeRetain<CPDF_Array>();
-    func_array->Append(CreateType0FunctionStream());
+    func_array->Append(CreateType0FunctionStreamReference(holder));
     func_array->Append(CreateType2FunctionDict());
-    func_array->Append(CreateBadType4FunctionStream());
+    auto func_stream = CreateBadType4FunctionStream();
+    const int func_stream_object_number =
+        holder.AddIndirectObject(std::move(func_stream));
+    func_array->Append(
+        pdfium::MakeRetain<CPDF_Reference>(&holder, func_stream_object_number));
 
     TestDocRenderData render_data;
     auto func = render_data.CreateTransferFuncForTesting(func_array);
