@@ -8,6 +8,7 @@
 
 #include <limits>
 #include <memory>
+#include <utility>
 
 #include "core/fpdfapi/page/cpdf_form.h"
 #include "core/fpdfapi/page/cpdf_pageimagecache.h"
@@ -17,6 +18,7 @@
 #include "core/fpdfapi/render/cpdf_renderoptions.h"
 #include "core/fpdfapi/render/cpdf_renderstatus.h"
 #include "core/fxcrt/fx_safe_types.h"
+#include "core/fxcrt/span_util.h"
 #include "core/fxge/cfx_defaultrenderdevice.h"
 #include "core/fxge/dib/cfx_dibitmap.h"
 
@@ -59,11 +61,11 @@ RetainPtr<CFX_DIBitmap> DrawPatternBitmap(
   context.AppendLayer(pPatternForm, mtPattern2Bitmap);
   context.Render(&bitmap_device, nullptr, &options, nullptr);
 
-#if defined(_SKIA_SUPPORT_)
+#if defined(PDF_USE_SKIA)
   if (CFX_DefaultRenderDevice::UseSkiaRenderer()) {
     pBitmap->UnPreMultiply();
   }
-#endif  // defined(_SKIA_SUPPORT_)
+#endif  // defined(PDF_USE_SKIA)
   return pBitmap;
 }
 
@@ -87,8 +89,8 @@ RetainPtr<CFX_DIBitmap> CPDF_RenderTiling::Draw(
   float ceil_width = std::ceil(cell_bbox.Width());
 
   // Validate the float will fit into the int when the conversion is done.
-  if (!pdfium::base::IsValueInRangeForNumericType<int>(ceil_height) ||
-      !pdfium::base::IsValueInRangeForNumericType<int>(ceil_width)) {
+  if (!pdfium::IsValueInRangeForNumericType<int>(ceil_height) ||
+      !pdfium::IsValueInRangeForNumericType<int>(ceil_width)) {
     return nullptr;
   }
 
@@ -230,8 +232,8 @@ RetainPtr<CFX_DIBitmap> CPDF_RenderTiling::Draw(
         uint32_t* dest_buf = reinterpret_cast<uint32_t*>(
             pScreen->GetWritableScanline(start_y).subspan(start_x * 4).data());
         if (pPattern->colored()) {
-          const auto* src_buf32 =
-              reinterpret_cast<const uint32_t*>(src_buf.data());
+          const uint32_t* src_buf32 =
+              fxcrt::reinterpret_span<const uint32_t>(src_buf).data();
           *dest_buf = *src_buf32;
         } else {
           *dest_buf = (*(src_buf.data()) << 24) | (fill_argb & 0xffffff);
