@@ -1,4 +1,4 @@
-// Copyright 2016 PDFium Authors. All rights reserved.
+// Copyright 2016 The PDFium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,8 +11,9 @@
 
 #include "core/fpdfapi/parser/cpdf_simple_parser.h"
 #include "core/fpdfapi/parser/fpdf_parser_utility.h"
+#include "core/fxcrt/fx_string.h"
+#include "core/fxcrt/notreached.h"
 #include "core/fxge/cfx_color.h"
-#include "third_party/base/notreached.h"
 
 namespace {
 
@@ -28,7 +29,7 @@ bool FindTagParamFromStart(CPDF_SimpleParser* parser,
   int buf_count = 0;
 
   parser->SetCurPos(0);
-  while (1) {
+  while (true) {
     pBuf[buf_index++] = parser->GetCurPos();
     if (buf_index == nParams)
       buf_index = 0;
@@ -53,8 +54,6 @@ bool FindTagParamFromStart(CPDF_SimpleParser* parser,
 
 }  // namespace
 
-CPDF_DefaultAppearance::CPDF_DefaultAppearance() = default;
-
 CPDF_DefaultAppearance::CPDF_DefaultAppearance(const ByteString& csDA)
     : m_csDA(csDA) {}
 
@@ -63,13 +62,14 @@ CPDF_DefaultAppearance::CPDF_DefaultAppearance(
 
 CPDF_DefaultAppearance::~CPDF_DefaultAppearance() = default;
 
-absl::optional<ByteString> CPDF_DefaultAppearance::GetFont(float* fFontSize) {
+std::optional<ByteString> CPDF_DefaultAppearance::GetFont(
+    float* fFontSize) const {
   *fFontSize = 0.0f;
   if (m_csDA.IsEmpty())
-    return absl::nullopt;
+    return std::nullopt;
 
   ByteString csFontNameTag;
-  CPDF_SimpleParser syntax(m_csDA.AsStringView().raw_span());
+  CPDF_SimpleParser syntax(m_csDA.AsStringView().unsigned_span());
   if (FindTagParamFromStart(&syntax, "Tf", 2)) {
     csFontNameTag = ByteString(syntax.GetWord());
     csFontNameTag.Delete(0, 1);
@@ -78,37 +78,36 @@ absl::optional<ByteString> CPDF_DefaultAppearance::GetFont(float* fFontSize) {
   return PDF_NameDecode(csFontNameTag.AsStringView());
 }
 
-absl::optional<CFX_Color> CPDF_DefaultAppearance::GetColor() const {
+std::optional<CFX_Color> CPDF_DefaultAppearance::GetColor() const {
   if (m_csDA.IsEmpty())
-    return absl::nullopt;
+    return std::nullopt;
 
-  float fc[4];
-  CPDF_SimpleParser syntax(m_csDA.AsStringView().raw_span());
+  CPDF_SimpleParser syntax(m_csDA.AsStringView().unsigned_span());
   if (FindTagParamFromStart(&syntax, "g", 1)) {
-    fc[0] = StringToFloat(syntax.GetWord());
-    return CFX_Color(CFX_Color::Type::kGray, fc[0]);
+    float gray = StringToFloat(syntax.GetWord());
+    return CFX_Color(CFX_Color::Type::kGray, gray);
   }
   if (FindTagParamFromStart(&syntax, "rg", 3)) {
-    fc[0] = StringToFloat(syntax.GetWord());
-    fc[1] = StringToFloat(syntax.GetWord());
-    fc[2] = StringToFloat(syntax.GetWord());
-    return CFX_Color(CFX_Color::Type::kRGB, fc[0], fc[1], fc[2]);
+    float r = StringToFloat(syntax.GetWord());
+    float g = StringToFloat(syntax.GetWord());
+    float b = StringToFloat(syntax.GetWord());
+    return CFX_Color(CFX_Color::Type::kRGB, r, g, b);
   }
   if (FindTagParamFromStart(&syntax, "k", 4)) {
-    fc[0] = StringToFloat(syntax.GetWord());
-    fc[1] = StringToFloat(syntax.GetWord());
-    fc[2] = StringToFloat(syntax.GetWord());
-    fc[3] = StringToFloat(syntax.GetWord());
-    return CFX_Color(CFX_Color::Type::kCMYK, fc[0], fc[1], fc[2], fc[3]);
+    float c = StringToFloat(syntax.GetWord());
+    float m = StringToFloat(syntax.GetWord());
+    float y = StringToFloat(syntax.GetWord());
+    float k = StringToFloat(syntax.GetWord());
+    return CFX_Color(CFX_Color::Type::kCMYK, c, m, y, k);
   }
-  return absl::nullopt;
+  return std::nullopt;
 }
 
-absl::optional<CFX_Color::TypeAndARGB> CPDF_DefaultAppearance::GetColorARGB()
+std::optional<CFX_Color::TypeAndARGB> CPDF_DefaultAppearance::GetColorARGB()
     const {
-  absl::optional<CFX_Color> maybe_color = GetColor();
+  std::optional<CFX_Color> maybe_color = GetColor();
   if (!maybe_color.has_value())
-    return absl::nullopt;
+    return std::nullopt;
 
   const CFX_Color& color = maybe_color.value();
   if (color.nColorType == CFX_Color::Type::kGray) {
@@ -133,10 +132,10 @@ absl::optional<CFX_Color::TypeAndARGB> CPDF_DefaultAppearance::GetColorARGB()
                    static_cast<int>(g * 255 + 0.5f),
                    static_cast<int>(b * 255 + 0.5f)));
   }
-  NOTREACHED();
-  return absl::nullopt;
+  NOTREACHED_NORETURN();
 }
 
+// static
 bool CPDF_DefaultAppearance::FindTagParamFromStartForTesting(
     CPDF_SimpleParser* parser,
     ByteStringView token,
