@@ -14,10 +14,10 @@
 #include "core/fpdfdoc/cpvt_word.h"
 #include "core/fpdfdoc/cpvt_wordinfo.h"
 #include "core/fpdfdoc/ipvt_fontmap.h"
+#include "core/fxcrt/check.h"
 #include "core/fxcrt/fx_codepage.h"
 #include "core/fxcrt/fx_safe_types.h"
 #include "core/fxcrt/stl_util.h"
-#include "third_party/base/check.h"
 
 namespace {
 
@@ -346,7 +346,7 @@ CPVT_WordPlace CPVT_VariableText::WordIndexToWordPlace(int32_t index) const {
       break;
     }
     if (nIndex > index) {
-      place.nSecIndex = pdfium::base::checked_cast<int32_t>(i);
+      place.nSecIndex = pdfium::checked_cast<int32_t>(i);
       place.nWordIndex = index - nOldIndex - 1;
       pSection->UpdateWordPlace(place);
       bFound = true;
