@@ -13,7 +13,7 @@
 #include <utility>
 
 #include "core/fxcrt/fx_safe_types.h"
-#include "third_party/base/memory/ptr_util.h"
+#include "core/fxcrt/ptr_util.h"
 
 namespace fxcodec {
 
