@@ -11,9 +11,9 @@
 
 #include "core/fxcodec/jbig2/JBig2_BitStream.h"
 #include "core/fxcodec/jbig2/JBig2_Context.h"
+#include "core/fxcrt/check.h"
 #include "core/fxcrt/fx_safe_types.h"
 #include "core/fxcrt/unowned_ptr_exclusion.h"
-#include "third_party/base/check.h"
 
 namespace {
 
@@ -135,7 +135,7 @@ CJBig2_HuffmanTable::CJBig2_HuffmanTable(size_t idx) {
   DCHECK(idx < kNumHuffmanTables);
   const HuffmanTable& table = kHuffmanTables[idx];
   HTOOB = table.HTOOB;
-  NTEMP = pdfium::base::checked_cast<uint32_t>(table.size);
+  NTEMP = pdfium::checked_cast<uint32_t>(table.size);
   m_bOK = ParseFromStandardTable(idx);
   DCHECK(m_bOK);
 }
