@@ -12,7 +12,8 @@
 // Prove consistency with incomplete forward definitions.
 #include "core/fxcrt/fx_codepage_forward.h"
 #include "core/fxcrt/fx_string.h"
-#include "third_party/base/span.h"
+#include "core/fxcrt/span.h"
+#include "core/fxcrt/unowned_ptr_exclusion.h"
 
 enum class FX_CodePage : uint16_t {
   kDefANSI = 0,
@@ -103,7 +104,7 @@ enum class FX_Charset : uint8_t {
 // Hi-bytes to unicode codepoint mapping for various code pages.
 struct FX_CharsetUnicodes {
   FX_Charset m_Charset;
-  const uint16_t* m_pUnicodes;  // Raw, POD struct.
+  UNOWNED_PTR_EXCLUSION const uint16_t* m_pUnicodes;  // POD struct.
 };
 
 extern const FX_CharsetUnicodes kFX_CharsetUnicodes[8];
