@@ -6,7 +6,10 @@
  */
 
 #include "orcus_test_global.hpp"
+#include "filesystem_env.hpp"
+
 #include "orcus/orcus_xls_xml.hpp"
+#include <orcus/format_detection.hpp>
 #include "orcus/stream.hpp"
 #include "orcus/config.hpp"
 #include <orcus/parser_global.hpp>
@@ -37,7 +40,7 @@ namespace {
 
 config test_config(format_t::xls_xml);
 
-constexpr std::string_view dirs[] = {
+const std::vector<fs::path> dirs = {
     SRCDIR"/test/xls-xml/basic/",
     SRCDIR"/test/xls-xml/basic-utf-16-be/",
     SRCDIR"/test/xls-xml/basic-utf-16-le/",
@@ -56,6 +59,7 @@ constexpr std::string_view dirs[] = {
     SRCDIR"/test/xls-xml/named-expression-sheet-local/",
     SRCDIR"/test/xls-xml/raw-values-1/",
     SRCDIR"/test/xls-xml/table-offset/",
+    SRCDIR"/test/xls-xml/unnamed-parent-styles/",
 };
 
 std::unique_ptr<spreadsheet::document> load_doc_from_filepath(
@@ -152,15 +156,42 @@ void update_config(spreadsheet::document& doc, const std::string& path)
     }
 }
 
+void test_xls_xml_detection()
+{
+    ORCUS_TEST_FUNC_SCOPE;
+
+    for (const auto& dir : dirs)
+    {
+        fs::path filepath = dir / "input.xml";
+        file_content fc(filepath.string());
+        assert(!fc.empty());
+
+        format_t detected = detect(fc.str());
+        assert(detected == format_t::xls_xml);
+    }
+}
+
+void test_xls_xml_create_filter()
+{
+    ORCUS_TEST_FUNC_SCOPE;
+
+    ss::range_size_t ssize{1048576, 16384};
+    std::unique_ptr<ss::document> doc = std::make_unique<ss::document>(ssize);
+    ss::import_factory factory(*doc);
+
+    auto f = create_filter(format_t::xls_xml, &factory);
+    assert(f);
+    assert(f->get_name() == "xls-xml");
+}
+
 void test_xls_xml_import()
 {
     ORCUS_TEST_FUNC_SCOPE;
 
-    auto verify = [](spreadsheet::document& doc, std::string_view dir)
+    auto verify = [](spreadsheet::document& doc, const fs::path& dir)
     {
-        std::string path{dir};
-        path.append("config.yaml");
-        update_config(doc, path);
+        auto path = dir / "config.yaml";
+        update_config(doc, path.string());
 
         // Dump the content of the model.
         std::ostringstream os;
@@ -168,9 +199,8 @@ void test_xls_xml_import()
         std::string check = os.str();
 
         // Check that against known control.
-        path = dir;
-        path.append("check.txt");
-        file_content control(path.data());
+        path = dir / "check.txt";
+        file_content control(path.string());
 
         assert(!check.empty());
         assert(!control.empty());
@@ -191,18 +221,16 @@ void test_xls_xml_import()
         }
     };
 
-    for (auto dir : dirs)
+    for (const auto& dir : dirs)
     {
         std::cout << dir << std::endl;
 
-        std::string path(dir);
-
         // Read the input.xml document.
-        path.append("input.xml");
-        std::unique_ptr<spreadsheet::document> doc = load_doc_from_filepath(path);
+        fs::path filepath = dir / "input.xml";
+        std::unique_ptr<spreadsheet::document> doc = load_doc_from_filepath(filepath.string());
         verify(*doc, dir);
 
-        doc = load_doc_from_stream(path);
+        doc = load_doc_from_stream(filepath.string());
         verify(*doc, dir);
     }
 }
@@ -347,10 +375,10 @@ void test_xls_xml_bold_and_italic()
     assert(sp);
     assert(*sp == "Bold and Italic mixed");
 
-    // The string contains 2 formatted segments.
+    // The string contains 4 formatted segments.
     const spreadsheet::format_runs_t* fmt_runs = ss.get_format_runs(si);
     assert(fmt_runs);
-    assert(fmt_runs->size() == 2);
+    assert(fmt_runs->size() == 4);
 
     // First formatted segment is bold.
     const spreadsheet::format_run* fmt_run = &fmt_runs->at(0);
@@ -359,8 +387,8 @@ void test_xls_xml_bold_and_italic()
     assert(fmt_run->bold);
     assert(!fmt_run->italic);
 
-    // Second formatted segment is italic.
-    fmt_run = &fmt_runs->at(1);
+    // Third formatted segment is italic.
+    fmt_run = &fmt_runs->at(2);
     assert(fmt_run->pos == 9);
     assert(fmt_run->size == 6);
     assert(!fmt_run->bold);
@@ -431,12 +459,12 @@ void test_xls_xml_colored_text()
     const spreadsheet::format_runs_t* fmt_runs = ss.get_format_runs(si);
     assert(fmt_runs);
 
-    // There should be 2 segments that are color-formatted.
-    assert(fmt_runs->size() == 2);
+    // There should be 2 segments that are color-formatted and one that is not.
+    assert(fmt_runs->size() == 3);
 
     // The 'Red' segment should be in red color.
     const spreadsheet::format_run* fmt = &fmt_runs->at(0);
-    assert(fmt->color.alpha == 0);
+    assert(fmt->color.alpha == 0xFF);
     assert(fmt->color.red == 0xFF);
     assert(fmt->color.green == 0);
     assert(fmt->color.blue == 0);
@@ -444,8 +472,8 @@ void test_xls_xml_colored_text()
     assert(fmt->size == 3);
 
     // The 'Blue' segment should be in blue color.
-    fmt = &fmt_runs->at(1);
-    assert(fmt->color.alpha == 0);
+    fmt = &fmt_runs->at(2);
+    assert(fmt->color.alpha == 0xFF);
     assert(fmt->color.red == 0);
     assert(fmt->color.green == 0x70);
     assert(fmt->color.blue == 0xC0);
@@ -453,6 +481,346 @@ void test_xls_xml_colored_text()
     assert(fmt->size == 4);
 }
 
+void test_xls_xml_formatted_text_basic()
+{
+    ORCUS_TEST_FUNC_SCOPE;
+
+    auto doc = load_doc_from_filepath(SRCDIR"/test/xls-xml/formatted-text/basic.xml");
+    const auto& styles_pool = doc->get_styles();
+
+    auto get_font = [&styles_pool](const ss::sheet& sh, ss::row_t row, ss::col_t col)
+    {
+        std::size_t xf = sh.get_cell_format(row, col);
+
+        const ss::cell_format_t* cell_format = styles_pool.get_cell_format(xf);
+        assert(cell_format);
+
+        const ss::font_t* font = styles_pool.get_font(cell_format->font);
+        assert(font);
+
+        return font;
+    };
+
+    auto check_cell_bold = [&get_font](const ss::sheet& sh, ss::row_t row, ss::col_t col, bool expected)
+    {
+        const ss::font_t* font = get_font(sh, row, col);
+
+        if (expected)
+        {
+            if (font->bold && *font->bold)
+                return true;
+
+            std::cerr << "expected to be bold but it is not "
+                << "(sheet=" << sh.get_index() << "; row=" << row << "; column=" << col << ")"
+                << std::endl;
+
+            return false;
+        }
+        else
+        {
+            if (!font->bold || !*font->bold)
+                return true;
+
+            std::cerr << "expected to be non-bold but it is bold "
+                << "(sheet=" << sh.get_index() << "; row=" << row << "; column=" << col << ")"
+                << std::endl;
+
+            return false;
+        }
+    };
+
+    auto check_cell_italic = [&get_font](const ss::sheet& sh, ss::row_t row, ss::col_t col, bool expected)
+    {
+        const ss::font_t* font = get_font(sh, row, col);
+
+        if (expected)
+        {
+            if (font->italic && *font->italic)
+                return true;
+
+            std::cerr << "expected to be italic but it is not "
+                << "(sheet=" << sh.get_index() << "; row=" << row << "; column=" << col << ")"
+                << std::endl;
+
+            return false;
+        }
+        else
+        {
+            if (!font->italic || !*font->italic)
+                return true;
+
+            std::cerr << "expected to be non-italic but it is italic "
+                << "(sheet=" << sh.get_index() << "; row=" << row << "; column=" << col << ")"
+                << std::endl;
+
+            return false;
+        }
+    };
+
+    auto check_cell_text = [&doc](const ss::sheet& sh, ss::row_t row, ss::col_t col, std::string_view expected)
+    {
+        const auto& sstrings = doc->get_shared_strings();
+
+        std::size_t si = sh.get_string_identifier(row, col);
+        const std::string* s = sstrings.get_string(si);
+        if (!s)
+        {
+            std::cerr << "expected='" << expected << "'; actual=<none> "
+                << "(sheet=" << sh.get_index() << "; row=" << row << "; column=" << col << ")"
+                << std::endl;
+
+            return false;
+        }
+
+        if (*s == expected)
+            return true;
+
+        std::cerr << "expected='" << expected << "'; actual='" << *s << "' "
+            << "(sheet=" << sh.get_index() << "; row=" << row << "; column=" << col << ")"
+            << std::endl;
+
+        return false;
+    };
+
+    {
+        const spreadsheet::sheet* sheet = doc->get_sheet("Text Properties");
+        assert(sheet);
+
+        ss::row_t row = 0;
+        ss::col_t col = 0;
+
+        // A1 - unformatted
+        assert(check_cell_text(*sheet, row, col, "Normal Text"));
+        assert(check_cell_bold(*sheet, row, col, false));
+        assert(check_cell_italic(*sheet, row, col, false));
+
+        // A2 - bold
+        row = 1;
+        assert(check_cell_text(*sheet, row, col, "Bold Text"));
+        assert(check_cell_bold(*sheet, row, col, true));
+        assert(check_cell_italic(*sheet, row, col, false));
+
+        // A3 - italic
+        row = 2;
+        assert(check_cell_text(*sheet, row, col, "Italic Text"));
+        assert(check_cell_bold(*sheet, row, col, false));
+        assert(check_cell_italic(*sheet, row, col, true));
+
+        // A4 - bold and italic
+        row = 3;
+        assert(check_cell_text(*sheet, row, col, "Bold and Italic Text"));
+        assert(check_cell_bold(*sheet, row, col, true));
+        assert(check_cell_italic(*sheet, row, col, true));
+
+        // A5 - bold and italic mixed. Excel creates format runs even for
+        // non-formatted segments.
+        row = 4;
+        assert(check_cell_text(*sheet, row, col, "Bold and Italic mixed"));
+
+        std::size_t si = sheet->get_string_identifier(row, col);
+        const ss::format_runs_t* runs = doc->get_shared_strings().get_format_runs(si);
+        assert(runs);
+        assert(runs->size() == 4u); // only 0 and 2 are formatted
+
+        // Bold and ...
+        // ^^^^
+        assert(runs->at(0).pos == 0);
+        assert(runs->at(0).size == 4);
+        assert(runs->at(0).bold);
+        assert(!runs->at(0).italic);
+
+        // Bold and Italic
+        //          ^^^^^^
+        assert(runs->at(2).pos == 9);
+        assert(runs->at(2).size == 6);
+        assert(!runs->at(2).bold);
+        assert(runs->at(2).italic);
+
+        // A6
+        row = 5;
+        assert(check_cell_text(*sheet, row, col, "Bold base with non-bold part"));
+        assert(check_cell_bold(*sheet, row, col, true));
+        assert(check_cell_italic(*sheet, row, col, false));
+
+        si = sheet->get_string_identifier(row, col);
+        runs = doc->get_shared_strings().get_format_runs(si);
+        assert(runs);
+        assert(runs->size() == 3u);
+
+        assert(runs->at(0).pos == 0);
+        assert(runs->at(0).size == 15);
+        assert(runs->at(0).bold);
+
+        assert(runs->at(1).pos == 15);
+        assert(runs->at(1).size == 8);
+        assert(!runs->at(1).bold);
+
+        assert(runs->at(2).pos == 23);
+        assert(runs->at(2).size == 5);
+        assert(runs->at(2).bold);
+
+        // A7 - TODO: check format
+        row = 6;
+        assert(check_cell_text(*sheet, row, col, "Only partially underlined"));
+
+        // A8
+        row = 7;
+        assert(check_cell_text(*sheet, row, col, "All Underlined"));
+        const ss::font_t* font = get_font(*sheet, row, col);
+        assert(font->underline_style);
+        assert(*font->underline_style == ss::underline_t::single_line);
+
+        // A9
+        row = 8;
+        assert(check_cell_text(*sheet, row, col, "Bold and underlined"));
+        assert(check_cell_bold(*sheet, row, col, true));
+        font = get_font(*sheet, row, col);
+        assert(font->underline_style);
+        assert(*font->underline_style == ss::underline_t::single_line);
+
+        row = 9;
+        assert(check_cell_text(*sheet, row, col, "All Strikethrough"));
+        // TODO: check for strikethrough in cell
+
+        // A11:A15 - TODO: check format
+        row = 10;
+        assert(check_cell_text(*sheet, row, col, "Partial strikethrough"));
+        row = 11;
+        assert(check_cell_text(*sheet, row, col, "Superscript"));
+        row = 12;
+        assert(check_cell_text(*sheet, row, col, "Subscript"));
+        row = 13;
+        assert(check_cell_text(*sheet, row, col, "x2 + y2 = 102"));
+        row = 14;
+        assert(check_cell_text(*sheet, row, col, "xi = yi + zi"));
+    }
+
+    {
+        const spreadsheet::sheet* sheet = doc->get_sheet("Fonts");
+        assert(sheet);
+
+        struct check
+        {
+            ss::row_t row;
+            std::string_view font_name;
+            double font_unit;
+        };
+
+        check checks[] = {
+            { 0, "Calibri Light", 12.0 },
+            { 1, "Arial", 18.0 },
+            { 2, "Times New Roman", 14.0 },
+            { 3, "Consolas", 9.0 },
+            { 4, "Bookman Old Style", 20.0 },
+        };
+
+        for (const auto& c : checks)
+        {
+            std::size_t xf = sheet->get_cell_format(c.row, 0);
+            const ss::cell_format_t* cell_format = styles_pool.get_cell_format(xf);
+            assert(cell_format);
+            const ss::font_t* font = styles_pool.get_font(cell_format->font);
+            assert(font);
+            assert(font->name == c.font_name);
+            assert(font->size == c.font_unit);
+
+            // Columns A and B should have the same font.
+            xf = sheet->get_cell_format(c.row, 1);
+            cell_format = styles_pool.get_cell_format(xf);
+            assert(cell_format);
+            font = styles_pool.get_font(cell_format->font);
+            assert(font);
+            assert(font->name == c.font_name);
+            assert(font->size == c.font_unit);
+        }
+    }
+
+    {
+        const spreadsheet::sheet* sheet = doc->get_sheet("Mixed Fonts");
+        assert(sheet);
+
+        // A1
+        ss::row_t row = 0;
+        ss::col_t col = 0;
+        assert(check_cell_text(*sheet, row, col, "C++ has class and struct as keywords."));
+
+        // Base cell has Serif 12-pt font applied
+        auto xf = sheet->get_cell_format(row, col);
+        const ss::cell_format_t* fmt = styles_pool.get_cell_format(xf);
+        assert(fmt);
+        const ss::font_t* font = styles_pool.get_font(fmt->font);
+        assert(font);
+        assert(font->name == "Calibri");
+        assert(font->size == 11.0);
+
+        // Two segments has Liberation Mono font applied (runs 1 and 3) whereas
+        // runs 0, 2 and 4 are unformatted.
+        std::size_t si = sheet->get_string_identifier(row, col);
+        const ss::format_runs_t* runs = doc->get_shared_strings().get_format_runs(si);
+        assert(runs);
+        assert(runs->size() == 5u);
+
+        // C++ has class ...
+        //         ^^^^^
+        assert(runs->at(1).pos == 8);
+        assert(runs->at(1).size == 5);
+        assert(runs->at(1).font == "Liberation Mono");
+
+        // ... and struct as ...
+        //         ^^^^^^
+        assert(runs->at(3).pos == 18);
+        assert(runs->at(3).size == 6);
+        assert(runs->at(3).font == "Liberation Mono");
+
+        // A2
+        row = 1;
+        assert(check_cell_text(*sheet, row, col, "Text with 12-point font, 24-point font, and 36-point font mixed."));
+        si = sheet->get_string_identifier(row, col);
+        runs = doc->get_shared_strings().get_format_runs(si);
+        assert(runs);
+        assert(runs->size() == 10u);
+
+        // with 12-point font, ...
+        //      ^^
+        assert(runs->at(1).pos == 10);
+        assert(runs->at(1).size == 2);
+        assert(runs->at(1).font_size == 12.0f);
+        assert(runs->at(1).color == ss::color_t(0xFF, 0xFF, 0, 0)); // red
+
+        // with 12-point font, ...
+        //        ^^^^^^
+        assert(runs->at(2).pos == 12);
+        assert(runs->at(2).size == 6);
+        assert(runs->at(2).font_size == 12.0f);
+
+        // 24-point font,
+        // ^^
+        assert(runs->at(4).pos == 25);
+        assert(runs->at(4).size == 2);
+        assert(runs->at(4).font_size == 24.0f);
+        assert(runs->at(4).color == ss::color_t(0xFF, 0xFF, 0, 0)); // red
+
+        // 24-point font,
+        //   ^^^^^^
+        assert(runs->at(5).pos == 27);
+        assert(runs->at(5).size == 6);
+        assert(runs->at(5).font_size == 24.0f);
+
+        // and 36-point font
+        //     ^^
+        assert(runs->at(7).pos == 44);
+        assert(runs->at(7).size == 2);
+        assert(runs->at(7).font_size == 36.0f);
+        assert(runs->at(7).color == ss::color_t(0xFF, 0xFF, 0, 0)); // red
+
+        // and 36-point font
+        //       ^^^^^^
+        assert(runs->at(8).pos == 46);
+        assert(runs->at(8).size == 6);
+        assert(runs->at(8).font_size == 36.0f);
+    }
+}
+
 void test_xls_xml_column_width_row_height()
 {
     ORCUS_TEST_FUNC_SCOPE;
@@ -473,50 +841,85 @@ void test_xls_xml_column_width_row_height()
 
     auto doc = load_doc_from_filepath(SRCDIR"/test/xls-xml/column-width-row-height/input.xml");
 
-    const spreadsheet::sheet* sheet1 = doc->get_sheet(0);
-    assert(sheet1);
-
     // Column widths and row heights are stored in twips. Convert them to
     // points so that we can compare them with the values stored in the source
     // file.
 
-    std::vector<cw_check> cw_checks =
     {
-        { 1, 56.25, 2 },
-        { 2, 82.50, 2 },
-        { 3, 108.75, 2 },
-        { 5, 66.75, 2 },
-        { 6, 66.75, 2 },
-        { 7, 66.75, 2 },
-        { 10, 119.25, 2 },
-        { 11, 119.25, 2 },
-    };
+        // Sheet1
+        const spreadsheet::sheet* sheet = doc->get_sheet(0);
+        assert(sheet);
 
-    for (const cw_check& check : cw_checks)
-    {
-        spreadsheet::col_width_t cw = sheet1->get_col_width(check.col, nullptr, nullptr);
-        double pt = convert(cw, length_unit_t::twip, length_unit_t::point);
-        test::verify_value_to_decimals(__FILE__, __LINE__, check.width, pt, check.decimals);
+        std::vector<cw_check> cw_checks =
+        {
+            { 1, 56.25, 2 },
+            { 2, 82.50, 2 },
+            { 3, 108.75, 2 },
+            { 5, 66.75, 2 },
+            { 6, 66.75, 2 },
+            { 7, 66.75, 2 },
+            { 10, 119.25, 2 },
+            { 11, 119.25, 2 },
+        };
+
+        for (const cw_check& check : cw_checks)
+        {
+            spreadsheet::col_width_t cw = sheet->get_col_width(check.col, nullptr, nullptr);
+            double pt = convert(cw, length_unit_t::twip, length_unit_t::point);
+            test::verify_value_to_decimals(__FILE__, __LINE__, check.width, pt, check.decimals);
+        }
+
+        std::vector<rh_check> rh_checks =
+        {
+            {  2, 20.0, 0 },
+            {  3, 30.0, 0 },
+            {  4, 40.0, 0 },
+            {  5, 50.0, 0 },
+            {  7, 25.0, 0 },
+            {  8, 25.0, 0 },
+            {  9, 25.0, 0 },
+            { 12, 35.0, 0 },
+            { 13, 35.0, 0 },
+        };
+
+        for (const rh_check& check : rh_checks)
+        {
+            spreadsheet::row_height_t rh = sheet->get_row_height(check.row, nullptr, nullptr);
+            double pt = convert(rh, length_unit_t::twip, length_unit_t::point);
+            test::verify_value_to_decimals(__FILE__, __LINE__, check.height, pt, check.decimals);
+        }
     }
 
-    std::vector<rh_check> rh_checks =
     {
-        {  2, 20.0, 0 },
-        {  3, 30.0, 0 },
-        {  4, 40.0, 0 },
-        {  5, 50.0, 0 },
-        {  7, 25.0, 0 },
-        {  8, 25.0, 0 },
-        {  9, 25.0, 0 },
-        { 12, 35.0, 0 },
-        { 13, 35.0, 0 },
-    };
+        // Sheet2
+        const spreadsheet::sheet* sheet = doc->get_sheet(1);
+        assert(sheet);
 
-    for (const rh_check& check : rh_checks)
-    {
-        spreadsheet::row_height_t rh = sheet1->get_row_height(check.row, nullptr, nullptr);
-        double pt = convert(rh, length_unit_t::twip, length_unit_t::point);
-        test::verify_value_to_decimals(__FILE__, __LINE__, check.height, pt, check.decimals);
+        std::vector<cw_check> cw_checks =
+        {
+            { 1, 119.25, 2 },
+            { 3, 234.75, 2 },
+        };
+
+        for (const cw_check& check : cw_checks)
+        {
+            spreadsheet::col_width_t cw = sheet->get_col_width(check.col, nullptr, nullptr);
+            double pt = convert(cw, length_unit_t::twip, length_unit_t::point);
+            test::verify_value_to_decimals(__FILE__, __LINE__, check.width, pt, check.decimals);
+        }
+
+        std::vector<rh_check> rh_checks =
+        {
+            {  2, 40.0, 0 },
+            {  4, 60.0, 0 },
+        };
+
+        for (const rh_check& check : rh_checks)
+        {
+            spreadsheet::row_height_t rh = sheet->get_row_height(check.row, nullptr, nullptr);
+            double pt = convert(rh, length_unit_t::twip, length_unit_t::point);
+            test::verify_value_to_decimals(__FILE__, __LINE__, check.height, pt, check.decimals);
+        }
     }
 }
 
@@ -1592,6 +1995,92 @@ void test_xls_xml_styles_column_styles()
     }
 }
 
+void test_xls_xml_styles_data_offset()
+{
+    ORCUS_TEST_FUNC_SCOPE;
+
+    std::string path{SRCDIR"/test/xls-xml/styles/data-offset.xml"};
+    auto doc = load_doc_from_filepath(path, false, ss::formula_error_policy_t::fail);
+    assert(doc);
+
+    const ss::styles& styles = doc->get_styles();
+
+    const ss::sheet* sh = doc->get_sheet(0);
+    assert(sh);
+
+    auto check_font_style1 = [sh, &styles](ss::row_t row, ss::col_t col)
+    {
+        std::size_t xfid = sh->get_cell_format(row, col);
+        const ss::cell_format_t* xf = styles.get_cell_format(xfid);
+        assert(xf);
+
+        const ss::font_t* font = styles.get_font(xf->font);
+        assert(font);
+        assert(font->bold);
+        assert(font->color);
+        assert(*font->color == ss::color_t(0xFF, 0x00, 0x80, 0x00));
+        const ss::number_format_t* numfmt = styles.get_number_format(xf->number_format);
+        assert(numfmt);
+        assert(numfmt->format_string);
+        assert(*numfmt->format_string == "0.00_ ;[Red]\\-0.00\\ ");
+    };
+
+    auto check_font_style2 = [sh, &styles](ss::row_t row, ss::col_t col)
+    {
+        const ss::color_t red{0xFF, 0xFF, 0x00, 0x00};
+
+        std::size_t xfid = sh->get_cell_format(row, col);
+        const ss::cell_format_t* xf = styles.get_cell_format(xfid);
+        assert(xf);
+
+        const ss::font_t* font = styles.get_font(xf->font);
+        assert(font);
+        assert(font->color);
+        assert(*font->color == red);
+    };
+
+    auto check_font_style3 = [sh, &styles](ss::row_t row, ss::col_t col)
+    {
+        const ss::color_t blue{0xFF, 0x00, 0x00, 0xFF};
+
+        std::size_t xfid = sh->get_cell_format(row, col);
+        const ss::cell_format_t* xf = styles.get_cell_format(xfid);
+        assert(xf);
+
+        const ss::font_t* font = styles.get_font(xf->font);
+        assert(font);
+        assert(font->color);
+        assert(*font->color == blue);
+
+        const ss::number_format_t* numfmt = styles.get_number_format(xf->number_format);
+        assert(numfmt);
+        assert(numfmt->format_string);
+        assert(*numfmt->format_string == "yyyy/mm\\-dd;@");
+    };
+
+    // Column B and row 2 should have font with bold, green-ish with number format applied
+    check_font_style1(0, 1);
+    check_font_style1(1, 1);
+    check_font_style1(2, 1);
+    check_font_style1(3, 1);
+
+    // row 2
+    check_font_style1(1, 0);
+    check_font_style1(1, 1);
+    check_font_style1(1, 2);
+    check_font_style1(1, 3);
+
+    // Column C should have red font (except for row 2)
+    check_font_style2(0, 2);
+    check_font_style2(2, 2);
+    check_font_style2(3, 2);
+
+    // Column D should have blue font with custom number format applied (except for row 2)
+    check_font_style3(0, 3);
+    check_font_style3(2, 3);
+    check_font_style3(3, 3);
+}
+
 void test_xls_xml_view_cursor_per_sheet()
 {
     ORCUS_TEST_FUNC_SCOPE;
@@ -1926,11 +2415,14 @@ int main()
     test_config.debug = false;
     test_config.structure_check = true;
 
+    test_xls_xml_detection();
+    test_xls_xml_create_filter();
     test_xls_xml_import();
     test_xls_xml_merged_cells();
     test_xls_xml_date_time();
     test_xls_xml_bold_and_italic();
     test_xls_xml_colored_text();
+    test_xls_xml_formatted_text_basic();
     test_xls_xml_column_width_row_height();
     test_xls_xml_background_fill();
     test_xls_xml_named_colors();
@@ -1946,6 +2438,7 @@ int main()
     test_xls_xml_cell_properties_locked_and_hidden();
     test_xls_xml_styles_direct_format();
     test_xls_xml_styles_column_styles();
+    test_xls_xml_styles_data_offset();
 
     // view import
     test_xls_xml_view_cursor_per_sheet();
